#include <curveFitting.h>

double capacitancePerTime = 203.19; // proportionality constant for t=kC [pF/us]
double capacitanceOffset = -3058; // offset for model

int insertPin = 2;
int chargePin = 3;
int dischargePin = 4;
int measurePin = 5;

volatile bool capCharging = false;
bool measuringCap = false;

bool calibrated = false;
int controlCapToMeasure = 0;
// Edit the line below with your own control caps
double control_caps[5] = {1000, 4700, 10000, 47000, 100000}; // in pF
double control_charge_times[5]; // in us

void setup() {
  // Set pins
  pinMode(chargePin, OUTPUT);
  pinMode(dischargePin, OUTPUT);
  pinMode(insertPin, INPUT);
  pinMode(measurePin, INPUT);
  attachInterrupt(digitalPinToInterrupt(insertPin), onCapCharging, FALLING);
  digitalWrite(chargePin, HIGH);
  digitalWrite(dischargePin, LOW);
   
  Serial.begin(9600);

  // Check if calibrated
  if(capacitancePerTime == 0) {
    calibrated = false;
    Serial.println("Please calibrate by measuring the following control capacitors");
    for (int i = 0; i < sizeof(control_caps)/sizeof(double); i++){
          Serial.println(control_caps[i]);
      }
    Serial.print("Please insert "); Serial.print(control_caps[controlCapToMeasure]); Serial.println("pF capacitor");
  } else {
    calibrated = true;
  }

}

void calibrate() {
  int order = 1;
  double coeffs[order + 1];

  int ret = fitCurve(order, sizeof(control_caps)/sizeof(double), control_charge_times, control_caps, sizeof(coeffs)/sizeof(double), coeffs);
  
  if (ret == 0){ //Returned value is 0 if no error
      uint8_t c = 'a';
      Serial.println("Coefficients are:");
      for (int i = 0; i < sizeof(coeffs)/sizeof(double); i++){
          Serial.println(coeffs[i]);
      }
  }

  capacitancePerTime = coeffs[0];
  capacitanceOffset = coeffs[1];
}

void onCapCharging() {
  capCharging = true;
}

int measureCapDischargeTime() {
  Serial.println("Measuring capacitance discharge time");
  int start = micros();
  //  Turn off charge circuit and turn on discharge circuit at same time
  PORTD ^= ((1 << chargePin) | (1 << dischargePin));
  
  while(bitRead(PIND, measurePin) == HIGH  ) {
    continue;
  }

  int timeTaken = micros() - start;
  
  //  Pull chargePin1 high again
  PORTD ^= ((1 << chargePin) | (1 << dischargePin));
  
  // Allow to charge up
//  delay(1000);

  return timeTaken;
}

void printCapacitance(int timeTaken) {
  Serial.print("Time taken: ");
  Serial.print(timeTaken);
  Serial.println("μs");
  Serial.print("Capacitance: " );
  Serial.print(timeTaken * capacitancePerTime + capacitanceOffset);
  Serial.println("pF");
}

void loop() {
  if(capCharging == true){ 
    Serial.println("Cap is charging");
    if(measuringCap == false) {
      measuringCap = true;
      delay(3000);
      capCharging = false;
      int result = measureCapDischargeTime(); 

      if(calibrated == true) {
        printCapacitance(result);  
      } else {
        Serial.print("CapToMeasure: "); Serial.println(controlCapToMeasure);
        if(controlCapToMeasure < sizeof(control_caps)/sizeof(double)) {
          control_charge_times[controlCapToMeasure] = result;
          controlCapToMeasure++;
        }
        
        if(controlCapToMeasure >= (sizeof(control_caps)/sizeof(double))) {
          calibrated = true;
          for (int i = 0; i < sizeof(control_caps)/sizeof(double); i++){
              Serial.print(control_caps[i]);   Serial.print(" pF| "); Serial.print(control_charge_times[i]); Serial.println(" us");
          }
          
          Serial.println("The model has been calibrated with the following parameters (x^1, x^0):");
          calibrate();
        } else {
          Serial.print("Please insert "); Serial.print(control_caps[controlCapToMeasure]); Serial.println("pF capacitor");
        }
        
      }
      
      measuringCap = false;
    }
    capCharging = false;
  }  
}
